% Decision rules for model A7 with N=4 countries

% Input:    state variables at time t (capital stocks and technology levels)
% Output:   decision variables: dv=[c;l;kp;i;lambda];
%               c:  Nx1 vector of consumption levels
%               l:  Nx1 vector of labor supplies
%               kp: Nx1 vector of next period capital stocks
%               i:  Nx1 vector of investment
%               lambda:  marginal utility of consumption
            
% Note: Model A1 features inelastic labor supply equal to one

function dv=policies_A7N4(states,dr_coeff_A7N4,kmin,kmax,amin,amax,mumin,mumax,regvec_labor)

k1=states(1);
k2=states(2);
k3=states(3);
k4=states(4);

a1=states(5);
a2=states(6);
a3=states(7);
a4=states(8);

dr_k1=dr_coeff_A7N4(0*end/5+1:1*end/5);
dr_k2=dr_coeff_A7N4(1*end/5+1:2*end/5);
dr_k3=dr_coeff_A7N4(2*end/5+1:3*end/5);
dr_k4=dr_coeff_A7N4(3*end/5+1:4*end/5);
dr_mu=dr_coeff_A7N4(4*end/5+1:5*end/5);

% 1) Calibration
N=4;
beta=0.99;
alpha=0.36;
delta=0.025;
sigma=0.01;
rho=0.95;
phi=0.5;
A=(1-beta)/(alpha*beta);
gammalow=0.25;
gammahigh=1;
gamma=[gammalow:(gammahigh-gammalow)/(N-1):gammahigh];
Le=2.5;
psi=(Le-alpha*(Le-1))^(-1);

% 2) Compute the steady state and the Negishi weights
kss=1;
lss=1;
Kss=N*kss;
Lss=N*lss;
css=A*kss^alpha*lss^(1-alpha);
Css=N*css;
lnass=0;
ass=1;
thetass=1;
Yss=A*Kss^alpha*Lss^(1-alpha);


% b) Compute Negishi weights
tau=zeros(1,N);
for j=1:N
    tau(j)=[psi*[[css^psi*(Le-lss)^(1-psi)]^(1-1/gamma(j))]/css]^(-1);
end
muss=1;

% 4) Scale down states to interval [-1,1] and create Chebyshev polynomials;
sd_k1=2*(k1-kmin)/(kmax-kmin)-1;
sd_k2=2*(k2-kmin)/(kmax-kmin)-1;
sd_k3=2*(k3-kmin)/(kmax-kmin)-1;
sd_k4=2*(k4-kmin)/(kmax-kmin)-1;

sd_a1=2*(a1-amin)/(amax-amin)-1;
sd_a2=2*(a2-amin)/(amax-amin)-1;
sd_a3=2*(a3-amin)/(amax-amin)-1;
sd_a4=2*(a4-amin)/(amax-amin)-1;

Cheby_k1=[1 sd_k1 2*(sd_k1^2)-1];
Cheby_k2=[1 sd_k2 2*(sd_k2^2)-1];
Cheby_k3=[1 sd_k3 2*(sd_k3^2)-1];
Cheby_k4=[1 sd_k4 2*(sd_k4^2)-1];

Cheby_a1=[1 sd_a1 2*(sd_a1^2)-1];
Cheby_a2=[1 sd_a2 2*(sd_a2^2)-1];
Cheby_a3=[1 sd_a3 2*(sd_a3^2)-1];
Cheby_a4=[1 sd_a4 2*(sd_a4^2)-1];

% 5) Create complete polynomial matrix 
NoSt=2*N;

X1=Cheby_k1;
X2=Cheby_k2;
X3=Cheby_k3;
X4=Cheby_k4;

X5=Cheby_a1;
X6=Cheby_a2;
X7=Cheby_a3;
X8=Cheby_a4;

REG=zeros(size(X1,1),1+NoSt+(NoSt-1)*NoSt/2+NoSt);
jjj=1;
REG(:,jjj)=ones(size(X1,1),1);
for i=1:NoSt
    jjj=jjj+1;
    XX=eval(['X' num2str(i)]);
    REG(:,jjj)=XX(:,2);
end
for j=1:(NoSt-1)
    for i=(j+1):NoSt
        jjj=jjj+1;
        XXj=eval(['X' num2str(j)]);
        XXi=eval(['X' num2str(i)]);
        REG(:,jjj)=XXj(:,2).*XXi(:,2);
    end
end
for i=1:NoSt
    jjj=jjj+1;
    XX=eval(['X' num2str(i)]);
    REG(:,jjj)=XX(:,3);
end

% 6) Compute decision variables
    
    % Next period's capital stocks
    kp1=REG*dr_k1;
    kp2=REG*dr_k2;
    kp3=REG*dr_k3;
    kp4=REG*dr_k4;
    kp_vec=[kp1;kp2;kp3;kp4];
    Kp=kp1+kp2+kp3+kp4;
    K=k1+k2+k3+k4;
    
    % Investment
    i1=kp1-(1-delta)*k1;
    i2=kp2-(1-delta)*k2;
    i3=kp3-(1-delta)*k3;
    i4=kp4-(1-delta)*k4;
    i_vec=[i1;i2;i3;i4];
    
    % Capital adjustment costs
    cac1=phi/2*k1.*(kp1./k1-1).^2;
    cac2=phi/2*k2.*(kp2./k2-1).^2;
    cac3=phi/2*k3.*(kp3./k3-1).^2;
    cac4=phi/2*k4.*(kp4./k4-1).^2;
    CAC=cac1+cac2+cac3+cac4;   
    
    % lambda=mu
    mu=exp(REG*dr_mu);
    lambda=mu;
    
    % Scale down mu
    sd_mu=2*(mu-mumin)/(mumax-mumin)-1;
       
    % Create Chebyhsev polynomials of order 8 to compute labor
    ord=8;
    
  	REG_K1=ones(1,ord+1);
    REG_K2=ones(1,ord+1);
    REG_K3=ones(1,ord+1);
    REG_K4=ones(1,ord+1);
    REG_A1=ones(1,ord+1);
    REG_A2=ones(1,ord+1);
    REG_A3=ones(1,ord+1);
    REG_A4=ones(1,ord+1);
    REG_MU=ones(1,ord+1);
    
    REG_K1(1,2)=sd_k1;
    REG_K2(1,2)=sd_k2;
    REG_K3(1,2)=sd_k3;
    REG_K4(1,2)=sd_k4;
    REG_A1(1,2)=sd_a1;
    REG_A2(1,2)=sd_a2;
    REG_A3(1,2)=sd_a3;
    REG_A4(1,2)=sd_a4;
    REG_MU(1,2)=sd_mu;
    
    for i=3:ord+1;
		REG_K1(1,i)=2.*sd_k1.*REG_K1(:,i-1)-REG_K1(:,i-2);
        REG_K2(1,i)=2.*sd_k2.*REG_K2(:,i-1)-REG_K2(:,i-2);
        REG_K3(1,i)=2.*sd_k3.*REG_K3(:,i-1)-REG_K3(:,i-2);
        REG_K4(1,i)=2.*sd_k4.*REG_K4(:,i-1)-REG_K4(:,i-2);
        REG_A1(1,i)=2.*sd_a1.*REG_A1(:,i-1)-REG_A1(:,i-2);
        REG_A2(1,i)=2.*sd_a2.*REG_A2(:,i-1)-REG_A2(:,i-2);
        REG_A3(1,i)=2.*sd_a3.*REG_A3(:,i-1)-REG_A3(:,i-2);
        REG_A4(1,i)=2.*sd_a4.*REG_A4(:,i-1)-REG_A4(:,i-2);
        REG_MU(1,i)=2.*sd_mu.*REG_MU(:,i-1)-REG_MU(:,i-2);
    end;
    
    counti=0;
    REGMAT1=zeros(size(REG_MU,1),200);
    REGMAT2=zeros(size(REG_MU,1),200);
    REGMAT3=zeros(size(REG_MU,1),200);
    REGMAT4=zeros(size(REG_MU,1),200);
        for i1=0:ord  
            for i2=0:[ord-i1];        
                for i3=0:[ord-i1-i2];
                    counti=counti+1;
                    REGMAT1(:,counti)=REG_MU(:,i1+1).*REG_A1(:,i2+1).*REG_K1(:,i3+1);
                    REGMAT2(:,counti)=REG_MU(:,i1+1).*REG_A2(:,i2+1).*REG_K2(:,i3+1);
                    REGMAT3(:,counti)=REG_MU(:,i1+1).*REG_A3(:,i2+1).*REG_K3(:,i3+1);
                    REGMAT4(:,counti)=REG_MU(:,i1+1).*REG_A4(:,i2+1).*REG_K4(:,i3+1);
                end
            end
        end

    REGMAT1=REGMAT1(:,1:counti);
    REGMAT2=REGMAT2(:,1:counti);
    REGMAT3=REGMAT3(:,1:counti);
    REGMAT4=REGMAT4(:,1:counti);
    
    l1=REGMAT1*regvec_labor(:,1);
    l2=REGMAT2*regvec_labor(:,2);
    l3=REGMAT3*regvec_labor(:,3);
    l4=REGMAT4*regvec_labor(:,4);
    l_vec=[l1;l2;l3;l4];
    
    % Consumption
    c1=[psi/(1-psi)].*(Le-l1).*a1.*(1-alpha).*A.*k1.^alpha.*l1.^(-alpha);
    c2=[psi/(1-psi)].*(Le-l2).*a2.*(1-alpha).*A.*k2.^alpha.*l2.^(-alpha);
    c3=[psi/(1-psi)].*(Le-l3).*a3.*(1-alpha).*A.*k3.^alpha.*l3.^(-alpha);
    c4=[psi/(1-psi)].*(Le-l4).*a4.*(1-alpha).*A.*k4.^alpha.*l4.^(-alpha);
    c_vec=[c1;c2;c3;c4];
 
% 7) Create output
dv=[c_vec;l_vec;kp_vec;i_vec;lambda];
